// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template file_selection_button_theme_extension}
/// File selection button theme extension that extends [ThemeExtension] and
/// implements copyWith, and lerp methods so you don't have to.
/// {@endtemplate}
class FileSelectionButtonThemeExtension
    extends ThemeExtension<FileSelectionButtonThemeExtension> {
  /// {@macro file_selection_button_theme_extension}
  const FileSelectionButtonThemeExtension({
    this.disabledStyle,
    this.focusedStyle,
    this.hoveredStyle,
    this.normalStyle,
    this.tappedStyle,
    this.selectedStyle,
    this.invalidStyle,
  });

  /// Style of this button in disabled state
  final FileSelectionButtonStyle? disabledStyle;

  /// Style of this button in focused state
  final FileSelectionButtonStyle? focusedStyle;

  /// Style of this button in hovered state
  final FileSelectionButtonStyle? hoveredStyle;

  /// Style of this button in normal state
  final FileSelectionButtonStyle? normalStyle;

  /// Style of this button in tapped state
  final FileSelectionButtonStyle? tappedStyle;

  /// Style of this button in selected state
  final FileSelectionButtonStyle? selectedStyle;

  /// Style of this button in invalid state
  final FileSelectionButtonStyle? invalidStyle;

  @override
  ThemeExtension<FileSelectionButtonThemeExtension> copyWith({
    FileSelectionButtonStyle? disabledStyle,
    FileSelectionButtonStyle? focusedStyle,
    FileSelectionButtonStyle? hoveredStyle,
    FileSelectionButtonStyle? normalStyle,
    FileSelectionButtonStyle? tappedStyle,
    FileSelectionButtonStyle? selectedStyle,
    FileSelectionButtonStyle? invalidStyle,
  }) =>
      FileSelectionButtonThemeExtension(
        disabledStyle: disabledStyle ?? this.disabledStyle,
        focusedStyle: focusedStyle ?? this.focusedStyle,
        hoveredStyle: hoveredStyle ?? this.hoveredStyle,
        normalStyle: normalStyle ?? this.normalStyle,
        tappedStyle: tappedStyle ?? this.tappedStyle,
        selectedStyle: selectedStyle ?? this.selectedStyle,
        invalidStyle: invalidStyle ?? this.invalidStyle,
      );

  @override
  ThemeExtension<FileSelectionButtonThemeExtension> lerp(
    covariant ThemeExtension<FileSelectionButtonThemeExtension>? other,
    double t,
  ) {
    if (other is! FileSelectionButtonThemeExtension) {
      return this;
    }
    return FileSelectionButtonThemeExtension(
      disabledStyle:
          FileSelectionButtonStyle.lerp(disabledStyle, other.disabledStyle, t),
      focusedStyle:
          FileSelectionButtonStyle.lerp(focusedStyle, other.focusedStyle, t),
      hoveredStyle:
          FileSelectionButtonStyle.lerp(hoveredStyle, other.hoveredStyle, t),
      normalStyle:
          FileSelectionButtonStyle.lerp(normalStyle, other.normalStyle, t),
      tappedStyle:
          FileSelectionButtonStyle.lerp(tappedStyle, other.tappedStyle, t),
      selectedStyle:
          FileSelectionButtonStyle.lerp(selectedStyle, other.selectedStyle, t),
      invalidStyle:
          FileSelectionButtonStyle.lerp(invalidStyle, other.invalidStyle, t),
    );
  }
}
